/*
** 1998-05-23 -	A big fat error handling module. There are *so* many possible errors,
**		I think we really must get a grip on that situation. This is it.
**		As far as grips go, I consider this to be pretty loose. :(
*/

#include "gentoo.h"

#include <ctype.h>
#include <stdarg.h>
#include <string.h>

#include "convutil.h"
#include "dirpane.h"
#include "dialog.h"
#include "guiutil.h"
#include "errors.h"

/* ----------------------------------------------------------------------------------------- */

/* Description of the error. This is in UTF-8. */
static gchar	error_desc[1024] = "";

/* ----------------------------------------------------------------------------------------- */

void err_clear(MainInfo *min)
{
	error_desc[0] = '\0';
	errno = 0;
}

/* 1999-01-21 -	Format an arbitrary string and print it on the status (error) line.
** UGLY   UGLY	Uses a sneaky little loop to ensure that the text is displayed *NOW*.
*/
void err_printf(MainInfo *min, const gchar *fmt, ...)
{
	va_list	al;

	va_start(al, fmt);
	vsnprintf(error_desc, sizeof error_desc, fmt, al);
	va_end(al);
	err_show(min);
	gui_events_flush();
}

/* 1998-05-30 -	Added special treatment of code == -1; this means that there is no
**		errno-compatible error information available, and the entire thing
**		is then suppressed.
*/
gint err_set(MainInfo *min, gint code, const gchar *source, const gchar *obj)
{
	gint	old_errno = errno;

	if(source != NULL)
	{
		Conv	conv;
		gchar	*src_display, *obj_display;

		/* Assume that the source and object names are in the filesystem encoding, and convert. */
		conv_init(&conv);
		src_display = conv_begin_reverse(&conv, source);
		obj_display = conv_add_reverse(&conv, obj);
		if(code >= 0)
		{
			const gchar	*desc;

			desc = g_strerror(code);
			if(obj != NULL)
				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\": %s (code %d)"), src_display, obj_display, desc, code);
			else
				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\" (code %d)"), src_display, obj_display, code);
		}
		else
		{
			if(obj != NULL)
				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\""), src_display, obj_display);
			else
				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s"), src_display);
		}
		conv_end(&conv);
	}
	errno = old_errno;
	return 1;
}

void err_show(MainInfo *min)
{
	if(error_desc[0] != '\0')
	{
		gtk_label_set_text(GTK_LABEL(min->gui->top), error_desc);
		gui_events_flush();
		if(min->cfg.errors.dialog)
			dlg_dialog_async_new_simple(error_desc, "Error", "OK", NULL, NULL);
		if(min->cfg.errors.beep)
			gdk_beep();
	}
	else
		dp_show_stats(min->gui->cur_pane);
}
